computeForageNesting <- function(landuseMap,edgesMap,unitEdges="m",widthEdges,guilds,codeEdges,cell.size=25,paramList,num_seasons=1,plotbool="TRUE"){
  
  #PURPOSE: 
  #This function maps the nesting resources and foraging resources available across the input landscape for the specified bird guilds. It is a minimally-adapted version of the 'computeFloralNesting.R' function supplied with the poll4pop model. 
  
  #INPUTS:
  #landuseMap = raster (must be square or rectangular in extent, with square pixels) denoting the base landcover type present in each pixel of the landscape over which bird populations are to be simulated. The landcover codes used in this raster must match those used in the input parameter file 'bird_habitat_params.csv'.
  #edgesMap = a stack of rasters, each with the same extent and resolution as the input landscape raster, which denote the area within each pixel that is covered by 'edge features'. These are small habitat features whose area occupies less than a full pixel (e.g. hedgerows, grassy field margins, etc.). There must be one raster per edge feature and the pixel values denote the area within each pixel that is covered by that edge feature in m2. 
  #unitEdges = the units used by the rasters in edgesMap. Set to "sqm" for the usage described in 'run_bird4pop.R'. 
  #widthEdges = a vector denoting the width of the edge features, to enable the function to calculate area covered by the edge feature when unitEdges="m" (implying the supplied edge feature rasters denote the length of edge feature in each pixel rather than the area covered by the edge feature in each pixel). Parameter is not used if unitEdges="sqm", so set to 1 for the usage described in 'run_bird4pop.R'. 
  #guilds = a vector giving the names of the bird guilds whose habitat resources are to be calculated, where the names must match those used in the input parameter file 'bird_guild_params.csv' and each guild must have a corresponding set of habitat use parameters assigned to it in the input parameter file 'bird_habitat_params.csv'.
  #codeEdges = a vector denoting the landcover type of each edge feature, where the codes used must match those used in the input parameter file 'bird_habitat_params.csv'. The order of the codes must match the order the edge features occur in edgestack.
  #cell.size = pixel size in meters of input landscape raster in meters, e.g. if input landscape has 10m x 10m pixels then cellsize should be set equal to 10. 
  #paramList = a list of the input parameters containing two items: the first item, 'guild_params', should contain the contents of the 'bird_guild_params.csv' file which is used by the function to determine the guild_code that corresponds to each requested bird guild. The second item, 'habitat_params', should contain the contents of the 'bird_habitat_params.csv' file which denotes the habitat resource scores for each landcover type for each bird guild. 
  #num_seasons = the number of foraging seasons to map, used for cases where there are seasonal changes in foraging resource availability/distribution. Set to 1 for the current single-season parameterisation of bird4pop. 
  #plotbool = if plotbool equals "TRUE" then the nesting and foraging resource maps for each guild are plotted. 
  
  #OUTPUTS:
  #The function returns a list. The first item, 'nest', contains the nesting resource maps (raster stack) for each requested bird guild. The second item, 'forage', contains the foraging resource maps (raster stack) for each requested bird guild.  
  
	
	require(raster)
  require(plyr)
  nr <- nrow(landuseMap)
  nc <- ncol(landuseMap)
  
  num_guilds <- length(guilds)
  #print(num_guilds)
  #print(num_seasons)
  
  #Create arrays
  emptyraster <- edgesMap[[1]]
  values(emptyraster) <- NA
  #print(emptyraster)
  nest <- mget(rep("emptyraster",num_guilds))
  names(nest) = paste(guilds,"nesting",sep="_")
  forage_seasons <- stack(mget(rep("emptyraster",num_seasons)))
  names(forage_seasons) = paste("forage_season",1:num_seasons,sep="_")
  #print(forage_seasons)
  forage <- mget(rep("forage_seasons",num_guilds))
  names(forage) = guilds
  #print(forage)
  attract = emptyraster
  #print(attract)
  
  #Get proportion of edge features in each pixel
  if(unitEdges=="sqm"){
    propEdges <- edgesMap/(cell.size^2)
  }
  if(unitEdges=="m"){
    propEdges <- edgesMap*widthEdges/(cell.size^2)
  }
  #print(sum(propEdges))
  
  #Do each guild
  for (s in 1:num_guilds)
    {print(guilds[s])
  	
  	#Get code for guild
  	guild_code <- paramList$guild_params$guild[match(guilds[s],as.character(paramList$guild_params$guild_names))]
    print(guild_code)
  	
    #Generate nesting resources map:
  	#Nesting resources from base landcover
    values(nest[[s]]) <- mapvalues(values(landuseMap),paramList$habitat_params[paramList$habitat_params[,'guild']==guild_code,'Code_ExpOp'],paramList$habitat_params[paramList$habitat_params[,'guild']==guild_code,"nesting"],warn_missing=FALSE)
    #Account for nesting resources from edge features
	  intermediateraster <- nest[[s]] * (1-sum(propEdges)) + sum(propEdges * paramList$habitat_params[paramList$habitat_params[,'guild']==guild_code,][pmatch(codeEdges,paramList$habitat_params[paramList$habitat_params[,'guild']==guild_code,][,'Code_ExpOp']),"nesting"])
	  values(nest[[s]]) = values(intermediateraster)
	  #print(paramList$habitat_params[paramList$habitat_params[,'guild']==guild_code,][pmatch(codeEdges,paramList$habitat_params[paramList$habitat_params[,'guild']==guild_code,][,'Code_ExpOp']),"nesting"])
	  #Set any negative values to zero
	  values(nest[[s]]) <- pmax(values(nest[[s]]),0)
	  #print(nest[[s]])
	  #print(paramList$habitat_params[paramList$habitat_params[,'guild']==guild_code,][pmatch(codeEdges,paramList$habitat_params[,'code']),"nesting"])
    
    #Generate foraging resources map for each season:
  	for (v in 1:num_seasons)
      {#Foraging resources from base landcover
  		values(attract) <- mapvalues(values(landuseMap),paramList$habitat_params[paramList$habitat_params[,'guild']==guild_code,'Code_ExpOp'],paramList$habitat_params[paramList$habitat_params[,'guild']==guild_code,paste0("foraging_P",v)],warn_missing=FALSE) 
	    #Get list of foraging attract value for each edge feature
	    attractEdges <- paramList$habitat_params[paramList$habitat_params[,'guild']==guild_code,][pmatch(codeEdges,paramList$habitat_params[paramList$habitat_params[,'guild']==guild_code,][,'Code_ExpOp']),paste0("foraging_P",v)]
	    #print(attractEdges)
	    #Account for foraging resources from edge features
	    values(forage[[s]][[v]]) <- values(attract * (1 - sum(propEdges)) + sum(propEdges * attractEdges))
	    #Set any negative values to zero
	    values(forage[[s]][[v]]) <- pmax(values(forage[[s]][[v]]),0)
    }
    
  }
  
  #Plot
  if (plotbool=="TRUE")
    {for (s in 1:length(guilds))
      {plot(nest[[s]],main=names(nest)[s]) 
      plot(forage[[s]],main=paste(names(forage)[s],"_forage",sep="")) 
    }
  }
	
	return(list(nest=nest,forage=forage))
	
}
