run_bird4pop <- function(fol_c,fol_ip,cellsize,landscape,edgesstack,edgecodes,guilds){
  
  #PURPOSE:
  #This is the top-level function used to run the bird4pop model.
  
  #INPUTS:
  #fol_c = file path to directory containing model codes
  #fol_ip = file path to directory containing input parameter files
  #cellsize = pixel size of input landscape raster in meters, e.g. if input landscape has 10m x 10m pixels then cellsize should be set equal to 10. 
  #landscape = raster (must be square or rectangular in extent, with square pixels) denoting the base landcover type present in each pixel of the landscape over which bird populations are to be simulated. The landcover codes used in this raster must match those used in the input parameter file 'bird_habitat_params.csv'.
  #edgestack = a stack of rasters, each with the same extent and resolution as the input landscape raster, which denote the area within each pixel that is covered by 'edge features'. These are small habitat features whose area occupies less than a full pixel (e.g. hedgerows, grassy field margins, etc.). There must be one raster per edge feature and the pixel values denote the area within each pixel that is covered by that edge feature in m2. 
  #edgecodes = a vector denoting the landcover type of each edge feature, where the codes used must match those used in the input parameter file 'bird_habitat_params.csv'. The order of the codes must match the order the edge features occur in edgestack. 
  #guilds = a vector giving the names of the bird guilds to be simulated where the names must match those used in the input parameter file 'bird_guild_params.csv' and each guild must have a corresponding set of habitat use parameters assigned to it in the input parameter file 'bird_habitat_params.csv'.
  
  #OUTPUTS:
  #output = a list with one item per bird guild simulated. Each item consists of a list of two further items - the outputs from running the computeForageNesting function for that guild and the outputs from running the getsteadystatepop function for that guild. 
  
  ##################################
  #SOURCE FUNCTIONS
  source(paste(fol_c,"computeForageNesting_v3.R",sep=""))
  source(paste(fol_c,"getsteadystatepop_v3.R",sep=""))
  source(paste(fol_c,"runpop_bird_v3.R",sep=""))
  source(paste(fol_c,"kerncalc.R",sep=""))
  source(paste(fol_c,"latfordisp.R",sep=""))
  source(paste(fol_c,"growth.func.R",sep=""))
  
  ##################################
  #LOAD PARAMETER FILES
  
  #Read in parameter csvs
  guild_params = read.csv(paste(fol_ip,"bird_guild_params.csv",sep=""),header=TRUE,stringsAsFactors=FALSE,na=c("",NA))
  habitat_params = read.csv(paste(fol_ip,"bird_habitat_params.csv",sep=""),header=TRUE,stringsAsFactors=FALSE,na=c("",NA))
  
  #Make parameter list
  parameters = list(guild_params=guild_params,habitat_params=habitat_params)
  #print(parameters)
  
  ##################################
  #RUN BIRD4POP
  
  #Make output list
  output = vector("list",length(guilds))
  
  #Do each guild
  for (s in 1:length(guilds))
    {print(guilds[s])
    birdsp = c(guilds[s])
    
    #Compute nesting and foraging resource maps
    nf = computeForageNesting(landuseMap=landscape,edgesMap=edgesstack,unitEdges="sqm",widthEdges=1,guilds=birdsp,codeEdges=edgecodes,cell.size=cellsize,paramList=parameters,num_seasons=1,plotbool=FALSE)
    
    #Compute steady state population
    pop = getsteadystatepop(guilds=birdsp,paramList=parameters,nf,cellsize,plotbool=TRUE)
    
    output[[s]] = list(nf,pop)
    
  }
  
  return(output)
  
}