runpop_bird <- function(P_pop0=numeric(0),nestlocs0=numeric(0),firstyearbool=TRUE,cellsize,guilds,paramList,resources){
  
  #PURPOSE:
  #This function simulates bird foraging and population processes to calculate, for a single year, for a given input landscape of habitat resources, likely nest locations, number of breeding pairs, foraging resources gathered, number of offspring produced, number of adults and juveniles surviving the subsequent non-breeding season given their respective yearly survival probabilities, and the distribution of breeding pairs after dispersal and re-pairing of surviving birds prior to the next breeding season. 
  
  #INPUTS:
  #P_pop0 = P_pop outputs from a previous run of 'runpop_bird' function. Only required if firstyearbool is not TRUE. 
  #nestlocs0 = nest locations from a previous run of 'runpop_bird'. Only required if firstyearbool is not TRUE.
  #firstyearbool = a boolean switch. If firstyearbool equals TRUE then the function probabilistically chooses nest site locations using the input nesting resources maps (obtained from 'resources') and the maximum nest density parameter (obtained from 'paramList') and initialises a breeding pair at each nest site location. If firstyearbool equals FALSE then the function uses the nest site locations given in 'nestlocs0' and the breeding pair locations given in 'P_pop0'. 
  #cellsize = pixel size in meters of the original landscape raster (and therefore also of the rasters in 'resources') in meters, e.g. if input landscape has 10m x 10m pixels then cellsize should be set equal to 10. 
  #guilds = a vector giving the names of the bird guilds to be simulated, where the names must match those used in the input parameter file 'bird_guild_params.csv'.
  #paramList = a list of the input parameters containing two items: the first item, 'guild_params', should contain the contents of the 'bird_guild_params.csv' file and the second item, 'habitat_params', should contain the contents of the 'bird_habitat_params.csv' file. 
  #resources = the outputs produced by the 'computeForageNesting' function, which is a list containing two items: the first item, 'nest', contains the nesting resource maps (raster stack) for each bird guild. The second item, 'forage', contains the foraging resource maps (raster stack) for each bird guild.
  
  #OUTPUTS:
  #This function returns a list of the following outputs:
  #P_pop = a stack of raster stacks storing the abundance of each bird guild (s) at each stage in the simulation process, which follows the convention:
      #P_pop[s][1] = no. of adult pairs in breeding season
      #P_pop[s][2] = no. of offspring fledged
      #P_pop[s][3] = no. of adults after probabilistic yearly mortality
      #P_pop[s][4] = no. of offspring after probabilistic yearly mortality
      #P_pop[s][5] = total no. of surviving birds after dispersal
      #P_pop[s][6] = no. of adult pairs before nest site limitation
      #P_pop[s][7] = no. of adult pairs after nest site limitation
  #forvis = a stack containing the 'adult foraging rate' raster for each bird guild simulated. 
  #resgath = a stack containing the 'foraging resources gathered' raster for each bird guild simulated. 
  #nestlocs = a stack containing the 'nest site locations' raster for each bird guild simulated. 
  
  require(raster)
  nr <- nrow(resources[[1]][[1]])
  nc <- ncol(resources[[1]][[1]])
  
  #Get number of guilds to model
  num_guilds <- length(guilds)
  guild_codes <- paramList$guild_params$guild[match(guilds,as.character(paramList$guild_params$guild_names))]
  #print(num_guilds)
  #print(guild_codes)
  
  #Create empty raster
  emptyraster <- resources[[1]][[1]]
  values(emptyraster) <- 0.0
  #print(emptyraster)
  
  #Create population size storage arrays for residents:
  P_pop <- stack(mget(rep("emptyraster",7)))
  names(P_pop) = c("breeding pairs","offspring fledged","adults after mort","offspring after mort","offspring after dispersal","pairs before nesting lim","pairs after nesting lim")
  P_pop <- mget(rep("P_pop",num_guilds))
  names(P_pop) = guilds
  #print(P_pop)
  
  #Create adult forage visitation rate storage arrays
  forvis <- stack(mget(rep("emptyraster",1)))
  names(forvis) = c("foraging")
  forvis <- mget(rep("forvis",num_guilds))
  names(forvis) = guilds
  #print(forvis)
  
  #Create adult resources gathered storage arrays (i.e. resource availability in the surrounding landscape for foragers originating from the focal cell)
  resgath <- stack(mget(rep("emptyraster",1)))
  names(resgath) = c("resourcesgathered")
  resgath <- mget(rep("resgath",num_guilds))
  names(resgath) = guilds
  #print(resgath)
  
  #Create nest locations storage arrays:
  nestlocs <- stack(mget(rep("emptyraster",num_guilds)))
  names(nestlocs) = paste(guilds,"nestlocs",sep="_")
  #print(nestlocs)
  
  #Compute kernels for breeding and dispersal ranges
  kernelB <- list()
  kernelN <- list()
  cutoff = 0.99
  for (s in 1:num_guilds)
    {kernelB[[s]] <- kerncalc((paramList$guild_params$range_breeding_m[guild_codes[s]]),cellsize,maxSize=floor(min((nr-1)/2,(nc-1)/2)),decaycut=cutoff)$decay
    kernelN[[s]] <- kerncalc((paramList$guild_params$range_dispersal_m[guild_codes[s]]),cellsize,maxSize=floor(min((nr-1)/2,(nc-1)/2)),decaycut=cutoff)$decay
    #print(paramList$guild_params$range_breeding_m[guild_codes[s]])
    #print(paramList$guild_params$range_dispersal_m[guild_codes[s]])
  }
  #print(kernelB)
  #print(kernelN)
  
  #Do each guild
  for (s in 1:num_guilds)
    {print(guilds[s])
    
    #If first year, define nest site locations and initialise pairs
    if (firstyearbool==TRUE)
      {values(nestlocs[[s]]) = rpois(length(values(resources$nest[[s]])),paramList$guild_params$maxnestdensity_perha[guild_codes[s]]*(cellsize*cellsize/10000.0)*values(resources$nest[[s]]))
      values(P_pop[[s]][[1]]) = values(nestlocs[[s]])
      #print(table(values(nestlocs[[s]])))
    }
    #Otherwise use previous year's nest site locations and pairs
    else
      {values(nestlocs[[s]]) = values(nestlocs0[[s]])
      values(P_pop[[s]][[1]]) = values(P_pop0[[s]][[7]])
    }
    print(c("BreedingPairs:",sum(values(P_pop[[s]][[1]]),na.rm=TRUE)))
    
    #Do breeding season:
    #Run breeding central-place foraging function for parent pairs
    foraging <- latfordisp(N=raster::as.matrix(P_pop[[s]][[1]]),alpha=raster::as.matrix(resources$forage[[s]][[1]]),decay=kernelB[[s]])
    #Fill in visitation rates to pixels surrounding the focal cells
    values(forvis[[s]][[1]]) <- as.vector(t(foraging$visitation_rate))
    #Fill in resources gathered by foragers nesting in the focal cells
    values(resgath[[s]][[1]]) <- as.vector(t(foraging$Racc_per_forager))
    #Produce chicks
    #a = resources needed to produce half maximum no. of chicks
    #b = curve steepness i.e. sensitivity to change in resources
    values(P_pop[[s]][[2]]) <- values(P_pop[[s]][[1]])*growth.func(R=values(resgath[[s]][[1]]),a=paramList$guild_params$growthparam_a[guild_codes[s]],b=paramList$guild_params$growthparam_b[guild_codes[s]],max=paramList$guild_params$chicksperyear_mean[guild_codes[s]])
    values(P_pop[[s]][[2]]) = round(values(P_pop[[s]][[2]]))
    #print(c(paramList$guild_params$growthparam_a[guild_codes[s]],paramList$guild_params$growthparam_b[guild_codes[s]],paramList$guild_params$maxbroodsize[guild_codes[s]]))
    #print(sum(values(P_pop[[s]][[2]])))
    print(c("FledgedOffspring:",sum(values(P_pop[[s]][[2]]),na.rm=TRUE)))
    
    #Reduce bird numbers by yearly survival probability:
    #Reduce adult numbers according to yearly survival probability
    indiv = 2.0*values(P_pop[[s]][[1]])
    survprob = paramList$guild_params$survprob_adult[guild_codes[s]]
    survindiv = numeric(length(indiv))
    for (i in 1:length(survindiv))
      {survindiv[i] = rbinom(n=1,size=indiv[i],prob=survprob)
    }
    values(P_pop[[s]][[3]]) <- survindiv
    #print(table(2.0*values(P_pop[[s]][[1]])))
    #print(table(values(P_pop[[s]][[3]])))
    #Reduce offspring numbers according to yearly survival probability
    indiv = values(P_pop[[s]][[2]])
    survprob = paramList$guild_params$survprob_juv[guild_codes[s]]
    survindiv = numeric(length(indiv))
    for (i in 1:length(survindiv))
      {survindiv[i] = rbinom(n=1,size=indiv[i],prob=survprob)
    }
    values(P_pop[[s]][[4]]) <- survindiv
    #print(table(round(values(P_pop[[s]][[2]]))))
    #print(table(values(P_pop[[s]][[4]])))
    print(c("AdultsAfterMort:",sum(values(P_pop[[s]][[3]]),na.rm=TRUE)))
    print(c("OffspringAfterMort:",sum(values(P_pop[[s]][[4]]),na.rm=TRUE)))
    
    #Pool surviving birds and re-disperse over available nest sites
    dispersing <- latfordisp(N=raster::as.matrix(P_pop[[s]][[3]]+P_pop[[s]][[4]]),alpha=raster::as.matrix(nestlocs[[s]]),decay=kernelN[[s]])
    values(P_pop[[s]][[5]]) = as.vector(t(dispersing$visitation_rate))
    print(c("DispersedSurvivingBirds:",sum(values(P_pop[[s]][[5]]),na.rm=TRUE)))
    
    #Pair up birds
    pairpotential = values(P_pop[[s]][[5]])/2.0
    wholepairs = floor(pairpotential)
    extrapairprob = pairpotential-wholepairs
    extrapair = rbinom(n=length(extrapairprob),size=1,extrapairprob)
    values(P_pop[[s]][[6]]) = wholepairs+extrapair
    print(c("PairsBeforeNestingLim:",sum(values(P_pop[[s]][[6]]),na.rm=TRUE)))
    #print(table(values(P_pop[[s]][[6]])))
    
    #Limit each pix to maximum nest density
    values(P_pop[[s]][[7]]) <- ifelse(values(P_pop[[s]][[6]])>values(nestlocs[[s]]),values(nestlocs[[s]]),values(P_pop[[s]][[6]]))
    #print(table(values(P_pop[[s]][[7]])))
    print(c("PairsAfterNestingLim:",sum(values(P_pop[[s]][[7]]),na.rm=TRUE)))
    
  }
  
  return(list(P_pop=P_pop,forvis=forvis,resgath=resgath,nestlocs=nestlocs))
  
}